@@ -1,5 +1,5 @@
 /*
- * $Header: /cvsroot/pgsql/contrib/pgbench/pgbench.c,v 1.22 2002/10/20 19:38:10 tgl Exp $
+ * $Header: /cvsroot/pgsql-server/contrib/pgbench/pgbench.c,v 1.27 2003/09/27 19:15:34 wieck Exp $
  *
  * pgbench: a simple TPC-B like benchmark program for PostgreSQL
  * written by Tatsuo Ishii
@@ -41,6 +41,10 @@
 #include <sys/resource.h>
 #endif   /* ! WIN32 */
 
+extern char *optarg;
+extern int	optind;
+
+
 /********************************************************************
  * some configurable parameters */
 
@@ -118,7 +122,7 @@ doConnect()
 {
 	PGconn	   *con;
 	PGresult   *res;
- 
+
 	con = PQsetdbLogin(pghost, pgport, pgoptions, pgtty, dbName,
 					   login, pwd);
 	if (con == NULL)
@@ -147,13 +151,6 @@ doConnect()
 		exit(1);
 	}
 	PQclear(res);
-	res = PQexec(con, "SET autocommit TO 'on'");
-	if (PQresultStatus(res) != PGRES_COMMAND_OK)
-	{
-		fprintf(stderr, "%s", PQerrorMessage(con));
-		exit(1);
-	}
-	PQclear(res);
 
 	return (con);
 }
@@ -570,7 +567,7 @@ init(void)
 			PQclear(res);
 		}
 
-		snprintf(sql, 256, "%d\t%d\t%d\t\n", j, j / naccounts, 0);
+		snprintf(sql, 256, "%d\t%d\t%d\t\n", j, i / naccounts + 1, 0);
 		if (PQputline(con, sql))
 		{
 			fprintf(stderr, "PQputline failed\n");
@@ -668,10 +665,6 @@ printResults(
 int
 main(int argc, char **argv)
 {
-	extern char *optarg;
-	extern int	optind,
-				opterr,
-				optopt;
 	int			c;
 	int			is_init_mode = 0;		/* initialize mode? */
 	int			is_no_vacuum = 0;		/* no vacuum at all before
@@ -701,6 +694,14 @@ main(int argc, char **argv)
 
 	PGconn	   *con;
 	PGresult   *res;
+	char	   *env;
+
+	if ((env = getenv("PGHOST")) != NULL && *env != '\0')
+		pghost = env;
+	if ((env = getenv("PGPORT")) != NULL && *env != '\0')
+		pgport = env;
+	else if ((env = getenv("PGUSER")) != NULL && *env != '\0')
+		login = env;
 
 	while ((c = getopt(argc, argv, "ih:nvp:dc:t:s:U:P:CNSl")) != -1)
 	{
@@ -795,8 +796,11 @@ main(int argc, char **argv)
 		dbName = argv[optind];
 	else
 	{
-		dbName = getenv("USER");
-		if (dbName == NULL)
+		if ((env = getenv("PGDATABASE")) != NULL && *env != '\0')
+			dbName = env;
+		else if (login != NULL && *login != '\0')
+			dbName = login;
+		else
 			dbName = "";
 	}
 
@@ -809,7 +813,7 @@ main(int argc, char **argv)
 	remains = nclients;
 
 	state = (CState *) malloc(sizeof(*state) * nclients);
-	memset(state, 0, sizeof(*state));
+	memset(state, 0, sizeof(*state) * nclients);
 
 	if (use_log)
 	{
