@@ -6,12 +6,12 @@
  *	  message integrity and endpoint authentication.
  *
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/interfaces/libpq/fe-secure.c,v 1.15.2.7 2003/08/22 21:57:44 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/interfaces/libpq/fe-secure.c,v 1.32 2003/09/29 16:38:04 petere Exp $
  *
  * NOTES
  *	  The client *requires* a valid server certificate.  Since
@@ -123,6 +123,7 @@
 
 #ifdef USE_SSL
 static int	verify_cb(int ok, X509_STORE_CTX *ctx);
+
 #ifdef NOT_USED
 static int	verify_peer(PGconn *);
 #endif
@@ -132,7 +133,7 @@ static DH  *tmp_dh_cb(SSL *s, int is_export, int keylength);
 static int	client_cert_cb(SSL *, X509 **, EVP_PKEY **);
 static int	initialize_SSL(PGconn *);
 static void destroy_SSL(void);
-static int	open_client_SSL(PGconn *);
+static PostgresPollingStatusType open_client_SSL(PGconn *);
 static void close_SSL(PGconn *);
 static const char *SSLerrmessage(void);
 #endif
@@ -156,6 +157,8 @@ static SSL_CTX *SSL_context = NULL;
  *	It's not critical that users have EPH keys, but it doesn't
  *	hurt and if it's missing someone will demand it, so....
  */
+#ifdef USE_SSL
+
 static const char file_dh512[] =
 "-----BEGIN DH PARAMETERS-----\n\
 MEYCQQD1Kv884bEpQBgRjXyEpwpy1obEAxnIByl6ypUM2Zafq9AKUJsCRtMIPWak\n\
@@ -193,6 +196,7 @@ OvOzKGtwcTqO/1wV5gKkzu1ZVswVUQd5Gg8lJicwqRWyyNRczDDoG9jVDxmogKTH\n\
 AaqLulO7R8Ifa1SwF2DteSGVtgWEN8gDpN3RBmmPTDngyF2DHb5qmpnznwtFKdTL\n\
 KWbuHn491xNO25CQWMtem80uKw+pTnisBRF/454n1Jnhub144YRBoN8CAQI=\n\
 -----END DH PARAMETERS-----\n";
+#endif
 
 /* ------------------------------------------------------------ */
 /*			 Procedures common to all secure sessions			*/
@@ -227,16 +231,30 @@ pqsecure_destroy(void)
 /*
  *	Attempt to negotiate secure session.
  */
-int
+PostgresPollingStatusType
 pqsecure_open_client(PGconn *conn)
 {
-	int			r = 0;
-
 #ifdef USE_SSL
-	r = open_client_SSL(conn);
+	/* First time through? */
+	if (conn->ssl == NULL)
+	{
+		if (!(conn->ssl = SSL_new(SSL_context)) ||
+			!SSL_set_app_data(conn->ssl, conn) ||
+			!SSL_set_fd(conn->ssl, conn->sock))
+		{
+			printfPQExpBuffer(&conn->errorMessage,
+			   libpq_gettext("could not establish SSL connection: %s\n"),
+							  SSLerrmessage());
+			close_SSL(conn);
+			return PGRES_POLLING_FAILED;
+		}
+	}
+	/* Begin or continue the actual handshake */
+	return open_client_SSL(conn);
+#else
+	/* shouldn't get here */
+	return PGRES_POLLING_FAILED;
 #endif
-
-	return r;
 }
 
 /*
@@ -262,43 +280,54 @@ pqsecure_read(PGconn *conn, void *ptr, size_t len)
 #ifdef USE_SSL
 	if (conn->ssl)
 	{
-	rloop:
+rloop:
 		n = SSL_read(conn->ssl, ptr, len);
 		switch (SSL_get_error(conn->ssl, n))
 		{
 			case SSL_ERROR_NONE:
 				break;
 			case SSL_ERROR_WANT_READ:
-				/* WANT_READ simply means no data available */
 				n = 0;
 				break;
 			case SSL_ERROR_WANT_WRITE:
-				/* XXX to support nonblock I/O, we should return 0 here */
+
+				/*
+				 * Returning 0 here would cause caller to wait for
+				 * read-ready, which is not correct since what SSL wants
+				 * is wait for write-ready.  The former could get us stuck
+				 * in an infinite wait, so don't risk it; busy-loop
+				 * instead.
+				 */
 				goto rloop;
 			case SSL_ERROR_SYSCALL:
-				if (n == -1)
-					printfPQExpBuffer(&conn->errorMessage,
-									  libpq_gettext("SSL SYSCALL error: %s\n"),
-									  SOCK_STRERROR(SOCK_ERRNO));
-				else
 				{
-					printfPQExpBuffer(&conn->errorMessage,
-									  libpq_gettext("SSL SYSCALL error: EOF detected\n"));
-					SOCK_ERRNO = ECONNRESET;
-					n = -1;
+					char		sebuf[256];
+
+					if (n == -1)
+						printfPQExpBuffer(&conn->errorMessage,
+								libpq_gettext("SSL SYSCALL error: %s\n"),
+						SOCK_STRERROR(SOCK_ERRNO, sebuf, sizeof(sebuf)));
+					else
+					{
+						printfPQExpBuffer(&conn->errorMessage,
+										  libpq_gettext("SSL SYSCALL error: EOF detected\n"));
+
+						SOCK_ERRNO_SET(ECONNRESET);
+						n = -1;
+					}
+					break;
 				}
-				break;
 			case SSL_ERROR_SSL:
 				printfPQExpBuffer(&conn->errorMessage,
 					  libpq_gettext("SSL error: %s\n"), SSLerrmessage());
 				/* fall through */
 			case SSL_ERROR_ZERO_RETURN:
-				SOCK_ERRNO = ECONNRESET;
+				SOCK_ERRNO_SET(ECONNRESET);
 				n = -1;
 				break;
 			default:
 				printfPQExpBuffer(&conn->errorMessage,
-								  libpq_gettext("Unknown SSL error code\n"));
+							  libpq_gettext("unrecognized SSL error code\n"));
 				n = -1;
 				break;
 		}
@@ -325,40 +354,51 @@ pqsecure_write(PGconn *conn, const void *ptr, size_t len)
 #ifdef USE_SSL
 	if (conn->ssl)
 	{
-	wloop:
 		n = SSL_write(conn->ssl, ptr, len);
 		switch (SSL_get_error(conn->ssl, n))
 		{
 			case SSL_ERROR_NONE:
 				break;
 			case SSL_ERROR_WANT_READ:
-			case SSL_ERROR_WANT_WRITE:
-				/* XXX to support nonblock I/O, we should return 0 here */
-				goto wloop;
-			case SSL_ERROR_SYSCALL:
-				if (n == -1)
-					printfPQExpBuffer(&conn->errorMessage,
-									  libpq_gettext("SSL SYSCALL error: %s\n"),
-									  SOCK_STRERROR(SOCK_ERRNO));
-				else
-				{
-					printfPQExpBuffer(&conn->errorMessage,
-									  libpq_gettext("SSL SYSCALL error: EOF detected\n"));
-					SOCK_ERRNO = ECONNRESET;
-					n = -1;
-				}
+
+				/*
+				 * Returning 0 here causes caller to wait for write-ready,
+				 * which is not really the right thing, but it's the best
+				 * we can do.
+				 */
+				n = 0;
 				break;
+			case SSL_ERROR_WANT_WRITE:
+				n = 0;
+				break;
+			case SSL_ERROR_SYSCALL:
+				{
+					char		sebuf[256];
+
+					if (n == -1)
+						printfPQExpBuffer(&conn->errorMessage,
+								libpq_gettext("SSL SYSCALL error: %s\n"),
+						SOCK_STRERROR(SOCK_ERRNO, sebuf, sizeof(sebuf)));
+					else
+					{
+						printfPQExpBuffer(&conn->errorMessage,
+										  libpq_gettext("SSL SYSCALL error: EOF detected\n"));
+						SOCK_ERRNO_SET(ECONNRESET);
+						n = -1;
+					}
+					break;
+				}
 			case SSL_ERROR_SSL:
 				printfPQExpBuffer(&conn->errorMessage,
 					  libpq_gettext("SSL error: %s\n"), SSLerrmessage());
 				/* fall through */
 			case SSL_ERROR_ZERO_RETURN:
-				SOCK_ERRNO = ECONNRESET;
+				SOCK_ERRNO_SET(ECONNRESET);
 				n = -1;
 				break;
 			default:
 				printfPQExpBuffer(&conn->errorMessage,
-								  libpq_gettext("Unknown SSL error code\n"));
+							  libpq_gettext("unrecognized SSL error code\n"));
 				n = -1;
 				break;
 		}
@@ -398,7 +438,6 @@ verify_cb(int ok, X509_STORE_CTX *ctx)
 #ifdef NOT_USED
 /*
  *	Verify that common name resolves to peer.
- *	This function is not thread-safe due to gethostbyname().
  */
 static int
 verify_peer(PGconn *conn)
@@ -414,9 +453,11 @@ verify_peer(PGconn *conn)
 	len = sizeof(addr);
 	if (getpeername(conn->sock, &addr, &len) == -1)
 	{
+		char		sebuf[256];
+
 		printfPQExpBuffer(&conn->errorMessage,
 						  libpq_gettext("error querying socket: %s\n"),
-						  SOCK_STRERROR(SOCK_ERRNO));
+						SOCK_STRERROR(SOCK_ERRNO, sebuf, sizeof(sebuf)));
 		return -1;
 	}
 
@@ -424,8 +465,23 @@ verify_peer(PGconn *conn)
 	if (addr.sa_family == AF_UNIX)
 		return 0;
 
+	{
+		struct hostent hpstr;
+		char		buf[BUFSIZ];
+		int			herrno = 0;
+		
+		/*
+		 *	Currently, pqGethostbyname() is used only on platforms that
+		 *	don't have getaddrinfo().  If you enable this function,
+		 *	you should convert the pqGethostbyname() function call to
+		 *	use getaddrinfo().
+		 */
+		pqGethostbyname(conn->peer_cn, &hpstr, buf, sizeof(buf),
+						&h, &herrno);
+	}
+
 	/* what do we know about the peer's common name? */
-	if ((h = gethostbyname(conn->peer_cn)) == NULL)
+	if (h == NULL)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
 		libpq_gettext("could not get information about host (%s): %s\n"),
@@ -447,7 +503,7 @@ verify_peer(PGconn *conn)
 
 		default:
 			printfPQExpBuffer(&conn->errorMessage,
-			  libpq_gettext("unsupported protocol\n"));
+							  libpq_gettext("unsupported protocol\n"));
 			return -1;
 	}
 
@@ -476,7 +532,7 @@ verify_peer(PGconn *conn)
 		default:
 			printfPQExpBuffer(&conn->errorMessage,
 							  libpq_gettext(
-			"server common name \"%s\" does not resolve to peer address\n"),
+											"server common name \"%s\" does not resolve to peer address\n"),
 							  conn->peer_cn);
 	}
 
@@ -494,18 +550,24 @@ verify_peer(PGconn *conn)
 static DH  *
 load_dh_file(int keylength)
 {
-	struct passwd *pwd;
+#ifdef WIN32
+    return NULL;
+#else
+	char		pwdbuf[BUFSIZ];
+	struct passwd pwdstr;
+	struct passwd *pwd = NULL;
 	FILE	   *fp;
 	char		fnbuf[2048];
 	DH		   *dh = NULL;
 	int			codes;
 
-	if ((pwd = getpwuid(getuid())) == NULL)
+	if (pqGetpwuid(getuid(), &pwdstr, pwdbuf, sizeof(pwdbuf), &pwd) == 0)
 		return NULL;
 
 	/* attempt to open file.  It's not an error if it doesn't exist. */
 	snprintf(fnbuf, sizeof fnbuf, "%s/.postgresql/dh%d.pem",
 			 pwd->pw_dir, keylength);
+
 	if ((fp = fopen(fnbuf, "r")) == NULL)
 		return NULL;
 
@@ -531,6 +593,7 @@ load_dh_file(int keylength)
 	}
 
 	return dh;
+#endif
 }
 
 /*
@@ -634,15 +697,22 @@ tmp_dh_cb(SSL *s, int is_export, int keylength)
 static int
 client_cert_cb(SSL *ssl, X509 **x509, EVP_PKEY **pkey)
 {
-	struct passwd *pwd;
+#ifdef WIN32
+   return 0;
+#else
+	char		pwdbuf[BUFSIZ];
+	struct passwd pwdstr;
+	struct passwd *pwd = NULL;
 	struct stat buf,
 				buf2;
 	char		fnbuf[2048];
 	FILE	   *fp;
 	PGconn	   *conn = (PGconn *) SSL_get_app_data(ssl);
 	int			(*cb) () = NULL;	/* how to read user password */
+	char		sebuf[256];
 
-	if ((pwd = getpwuid(getuid())) == NULL)
+
+	if (pqGetpwuid(getuid(), &pwdstr, pwdbuf, sizeof(pwdbuf), &pwd) == 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
 					  libpq_gettext("could not get user information\n"));
@@ -658,7 +728,7 @@ client_cert_cb(SSL *ssl, X509 **x509, EVP_PKEY **pkey)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
 				  libpq_gettext("could not open certificate (%s): %s\n"),
-						  fnbuf, strerror(errno));
+						  fnbuf, pqStrerror(errno, sebuf, sizeof(sebuf)));
 		return -1;
 	}
 	if (PEM_read_X509(fp, x509, NULL, NULL) == NULL)
@@ -686,7 +756,7 @@ client_cert_cb(SSL *ssl, X509 **x509, EVP_PKEY **pkey)
 		buf.st_uid != getuid())
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-						  libpq_gettext("private key (%s) has wrong permissions\n"), fnbuf);
+		libpq_gettext("private key (%s) has wrong permissions\n"), fnbuf);
 		X509_free(*x509);
 		return -1;
 	}
@@ -694,7 +764,7 @@ client_cert_cb(SSL *ssl, X509 **x509, EVP_PKEY **pkey)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
 			 libpq_gettext("could not open private key file (%s): %s\n"),
-						  fnbuf, strerror(errno));
+						  fnbuf, pqStrerror(errno, sebuf, sizeof(sebuf)));
 		X509_free(*x509);
 		return -1;
 	}
@@ -702,7 +772,7 @@ client_cert_cb(SSL *ssl, X509 **x509, EVP_PKEY **pkey)
 		buf.st_dev != buf2.st_dev || buf.st_ino != buf2.st_ino)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-			libpq_gettext("private key (%s) changed during execution\n"), fnbuf);
+						  libpq_gettext("private key (%s) changed during execution\n"), fnbuf);
 		X509_free(*x509);
 		return -1;
 	}
@@ -729,6 +799,7 @@ client_cert_cb(SSL *ssl, X509 **x509, EVP_PKEY **pkey)
 	}
 
 	return 1;
+#endif
 }
 
 /*
@@ -737,9 +808,13 @@ client_cert_cb(SSL *ssl, X509 **x509, EVP_PKEY **pkey)
 static int
 initialize_SSL(PGconn *conn)
 {
+#ifndef WIN32
 	struct stat buf;
-	struct passwd *pwd;
+	char		pwdbuf[BUFSIZ];
+	struct passwd pwdstr;
+	struct passwd *pwd = NULL;
 	char		fnbuf[2048];
+#endif
 
 	if (!SSL_context)
 	{
@@ -755,7 +830,8 @@ initialize_SSL(PGconn *conn)
 		}
 	}
 
-	if ((pwd = getpwuid(getuid())) != NULL)
+#ifndef WIN32
+	if (pqGetpwuid(getuid(), &pwdstr, pwdbuf, sizeof(pwdbuf), &pwd) == 0)
 	{
 		snprintf(fnbuf, sizeof fnbuf, "%s/.postgresql/root.crt",
 				 pwd->pw_dir);
@@ -763,17 +839,19 @@ initialize_SSL(PGconn *conn)
 		{
 			return 0;
 #ifdef NOT_USED
+			char		sebuf[256];
+
 			/* CLIENT CERTIFICATES NOT REQUIRED  bjm 2002-09-26 */
 			printfPQExpBuffer(&conn->errorMessage,
-				 libpq_gettext("could not read root certificate list (%s): %s\n"),
-							  fnbuf, strerror(errno));
+							  libpq_gettext("could not read root certificate list (%s): %s\n"),
+						 fnbuf, pqStrerror(errno, sebuf, sizeof(sebuf)));
 			return -1;
 #endif
 		}
 		if (!SSL_CTX_load_verify_locations(SSL_context, fnbuf, 0))
 		{
 			printfPQExpBuffer(&conn->errorMessage,
-				 libpq_gettext("could not read root certificate list (%s): %s\n"),
+							  libpq_gettext("could not read root certificate list (%s): %s\n"),
 							  fnbuf, SSLerrmessage());
 			return -1;
 		}
@@ -789,6 +867,7 @@ initialize_SSL(PGconn *conn)
 
 	/* set up mechanism to provide client certificate, if available */
 	SSL_CTX_set_client_cert_cb(SSL_context, client_cert_cb);
+#endif
 
 	return 0;
 }
@@ -809,29 +888,55 @@ destroy_SSL(void)
 /*
  *	Attempt to negotiate SSL connection.
  */
-static int
+static PostgresPollingStatusType
 open_client_SSL(PGconn *conn)
 {
-#ifdef NOT_USED
 	int			r;
-#endif
 
-	if (!(conn->ssl = SSL_new(SSL_context)) ||
-		!SSL_set_app_data(conn->ssl, conn) ||
-		!SSL_set_fd(conn->ssl, conn->sock) ||
-		SSL_connect(conn->ssl) <= 0)
+	r = SSL_connect(conn->ssl);
+	if (r <= 0)
 	{
-		printfPQExpBuffer(&conn->errorMessage,
-			   libpq_gettext("could not establish SSL connection: %s\n"),
-						  SSLerrmessage());
-		close_SSL(conn);
-		return -1;
+		switch (SSL_get_error(conn->ssl, r))
+		{
+			case SSL_ERROR_WANT_READ:
+				return PGRES_POLLING_READING;
+
+			case SSL_ERROR_WANT_WRITE:
+				return PGRES_POLLING_WRITING;
+
+			case SSL_ERROR_SYSCALL:
+				{
+					char		sebuf[256];
+
+					if (r == -1)
+						printfPQExpBuffer(&conn->errorMessage,
+								libpq_gettext("SSL SYSCALL error: %s\n"),
+						SOCK_STRERROR(SOCK_ERRNO, sebuf, sizeof(sebuf)));
+					else
+						printfPQExpBuffer(&conn->errorMessage,
+										  libpq_gettext("SSL SYSCALL error: EOF detected\n"));
+					close_SSL(conn);
+					return PGRES_POLLING_FAILED;
+				}
+			case SSL_ERROR_SSL:
+				printfPQExpBuffer(&conn->errorMessage,
+					  libpq_gettext("SSL error: %s\n"), SSLerrmessage());
+				close_SSL(conn);
+				return PGRES_POLLING_FAILED;
+
+			default:
+				printfPQExpBuffer(&conn->errorMessage,
+							  libpq_gettext("unrecognized SSL error code\n"));
+				close_SSL(conn);
+				return PGRES_POLLING_FAILED;
+		}
 	}
 
 	/* check the certificate chain of the server */
 
 #ifdef NOT_USED
 	/* CLIENT CERTIFICATES NOT REQUIRED  bjm 2002-09-26 */
+
 	/*
 	 * this eliminates simple man-in-the-middle attacks and simple
 	 * impersonations
@@ -843,7 +948,7 @@ open_client_SSL(PGconn *conn)
 			   libpq_gettext("certificate could not be validated: %s\n"),
 						  X509_verify_cert_error_string(r));
 		close_SSL(conn);
-		return -1;
+		return PGRES_POLLING_FAILED;
 	}
 #endif
 
@@ -855,7 +960,7 @@ open_client_SSL(PGconn *conn)
 				libpq_gettext("certificate could not be obtained: %s\n"),
 						  SSLerrmessage());
 		close_SSL(conn);
-		return -1;
+		return PGRES_POLLING_FAILED;
 	}
 
 	X509_NAME_oneline(X509_get_subject_name(conn->peer),
@@ -870,6 +975,7 @@ open_client_SSL(PGconn *conn)
 
 #ifdef NOT_USED
 	/* CLIENT CERTIFICATES NOT REQUIRED  bjm 2002-09-26 */
+
 	/*
 	 * this is necessary to eliminate man-in-the-middle attacks and
 	 * impersonations where the attacker somehow learned the server's
@@ -878,11 +984,12 @@ open_client_SSL(PGconn *conn)
 	if (verify_peer(conn) == -1)
 	{
 		close_SSL(conn);
-		return -1;
+		return PGRES_POLLING_FAILED;
 	}
 #endif
 
-	return 0;
+	/* SSL handshake is complete */
+	return PGRES_POLLING_OK;
 }
 
 /*
